SystemOrganization addCategory: #Loom!

TestCase subclass: #LoomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomTest subclass: #LoomAcquiringProcessTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringProcessTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 15:42'!
testSimulation

	| pool t1 t2 t3 |
	pool := LoomSimulatedchedulingStrategy new.
	pool add: (t1 := [] fork).
	pool add: (t2 := [] fork).
	pool add: (t3 := [] fork).
	
	self assert: pool getNextProcessToRun == t1.
	self assert: pool getNextProcessToRun == t2.
	self assert: pool getNextProcessToRun == t3.
	
	t1 terminate.
	t2 terminate.
	t3 terminate.! !

LoomTest subclass: #LoomAcquiringThreadStrategyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomTest subclass: #LoomEventTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomEventTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 12:42'!
testSampledEvent
	| t profiler |
	profiler := LoomProfiler new.
	t := 0.
	profiler on: LoomSamplingEvent do: [ :event | t := t + 1 ].
	profiler on: LoomSampledEvent do: [ :event | t := t + 10 ].
	
	self assert: t = 0.
	profiler sampleExecution.
	self assert: t = 11.! !

!LoomEventTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/11/2012 22:48'!
testSequence

	| event1 event2 |
	event1 := LoomSampledEvent new.
	(Delay forMilliseconds: 2) wait.
	event2 := LoomSampledEvent new.
	
	self assert: event1 time < event2 time! !

LoomTest subclass: #LoomPlatformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomPlatformTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 09:53'!
testLoomTime

	| time |
	time := LoomTime currentTimeAsMilliseconds.
	(Delay forMilliseconds: 2) wait.
	self assert: (LoomTime currentTimeAsMilliseconds - time ) >= 2! !

!LoomPlatformTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 10:00'!
testProcessor

	"Just to make sure we have what we expect"
	self assert: (Processor class == ProcessorScheduler)! !

LoomTest subclass: #LoomProfilerTest
	instanceVariableNames: 'thread1 thread2 profiler thread3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 14:23'!
isScenarioOver
	(Array with: thread1 with: thread2 with: thread3) 
		do: [ :t | (t notNil and: [ t isTerminated not ]) ifTrue: [ ^ false ] ].
	^ true
! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 14:46'!
runThread1
	thread1 := [ 200 timesRepeat: [ self wait ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 14:46'!
runThread2
	thread2 := [ 100 timesRepeat: [ self wait2 ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:38'!
runThread3
	thread3 := [ 50 timesRepeat: [ self wait3 ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:36'!
scenario1
	self runThread1.
	self runThread2! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:38'!
scenario2
	self runThread1.
	self runThread2.
	self runThread3! !

!LoomProfilerTest methodsFor: 'running' stamp: 'AlexandreBergel 9/5/2012 10:51'!
setUp
	super setUp.
	profiler := LoomProfiler new.! !

!LoomProfilerTest methodsFor: 'running' stamp: 'AlexandreBergel 9/12/2012 15:00'!
tearDown
	profiler stopProfiling.
	self waitForScenarioEnd! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 11:55'!
testBasic

	self scenario1.
	profiler profile: (Array with: thread1 with: thread2).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	self assert: profiler profilingTime > 10.
	self assert: profiler numberOfSamplings > 1.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."

	"self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread2)."
	self assert: (profiler consumptionTimeOf: thread1) >= 0.2.
	self assert: (profiler consumptionTimeOf: thread2) <= 0.7.! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 10:24'!
testBasic2

	self scenario2.
	profiler profile: (Array with: thread1 with: thread2 with: thread3).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."
	self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread2).
	self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread3).
	self assert: (profiler consumptionTimeOf: thread3) >  0.
	self assert: (profiler consumptionTimeOf: thread2) >  0! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 14:59'!
testDynamicCreationOfThreads

	| nbOriginal |
	nbOriginal := (LoomAllThreadsStrategy forProfiler: profiler) processesToProfile size.
	self runThread1.
	self runThread2.
	
	profiler profileAllProcesses.
	nbOriginal := profiler numberOfThreads.
	self runThread3.
	self wait2.
	self assert: (profiler numberOfThreads - nbOriginal) = 1.
	
	self waitForScenarioEnd! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 11:14'!
testGetLoomThreadFrom
	| p ps |
	p := [ ] fork.

	profiler profile: (Array with: p).
	self assert: (profiler getThreadFrom: p) process == p.
	self assert: (profiler getThreadFrom: p) id = 1.
	self assert: (profiler getThreadFrom: p) profiler == profiler! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 09:23'!
testGetLoomThreads
	"
	self debug: #testGetLoomThreads
	"
	| p ps |
	p := [ ] fork.

	ps := profiler getLoomThreadsFrom: (Array with: p).

	self assert: ps size = 1.
	self assert: ps first process == p.
	self assert: ps first id = 1.

	self assert: ps first profiler == profiler! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 14:50'!
testInitialization

	self assert: profiler numberOfThreads = 0! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/5/2012 11:40'!
testIsProfiling

	self deny: profiler isProfiling.
	profiler startProfiling.
	self assert: profiler isProfiling.
	
	profiler stopProfiling.
	self deny: profiler isProfiling.
	
! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 09:03'!
testPrintingThreads

	self shouldnt: [ profiler allCurrentThreads do: [ :t | t printString ] ] raise: Error! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 08:42'!
testPriority
	| p ps thread |
	p := [ ] fork.
	profiler profile: (Array with: p).
	thread := profiler getThreadFrom: p.

	self assert: thread priority = p priority! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 14:42'!
testThreadCreation

	| p ps t1 t2 |
	p := [ ] fork.

	profiler profile: (Array with: p).
	t1 := profiler getThreadFrom: p.
	t2 := profiler getThreadFrom: p.

	self assert: t1 process == t2 process.
	self assert: t1 == t2
	
! !

!LoomProfilerTest methodsFor: 'thread state' stamp: 'AlexandreBergel 9/6/2012 08:45'!
testThreadState

	| loomThread |
	self scenario1.
	profiler profile: (Array with: thread1).
	loomThread := profiler getThreadFrom: thread1. 
	
	self deny: thread1 isTerminated.
	self deny: loomThread isTerminated.
	
	self waitForScenarioEnd.
	
	self assert: thread1 isTerminated.
	self assert: loomThread isTerminated.
! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 09:55'!
wait
	(Delay forMilliseconds: 2) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 09:55'!
wait2
	(Delay forMilliseconds: 4) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 14:25'!
wait3
	(Delay forMilliseconds: 8) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 10:12'!
waitForScenarioEnd
	[ self isScenarioOver not ] whileTrue: [ self wait ]! !

Announcement subclass: #LoomEvent
	instanceVariableNames: 'time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomEvent class methodsFor: 'instance creation' stamp: 'AlexandreBergel 9/11/2012 22:43'!
new
	^ self basicNew initialize! !

!LoomEvent methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/11/2012 22:44'!
initialize
	time := LoomTime currentTimeAsMilliseconds! !

!LoomEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 9/11/2012 22:45'!
time
	^ time! !

LoomEvent subclass: #LoomSampledEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomEvent subclass: #LoomSamplingEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

Object subclass: #LoomObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomObject subclass: #LoomAcquiringProcessStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringProcessStrategy commentStamp: 'AlexandreBergel 9/12/2012 15:33' prior: 0!
LoomAcquiringProcessStrategy is a generic way to obtain Smalltalk process from the system. They may either be user selected, or programmatically selected.!

!LoomAcquiringProcessStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:37'!
getNextProcessToRun
	"Return the next Smalltalk process that will be run by the system scheduler"

	self subclassResponsibility ! !

LoomAcquiringProcessStrategy subclass: #LoomSimulatedchedulingStrategy
	instanceVariableNames: 'processesToDispatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSimulatedchedulingStrategy methodsFor: 'adding' stamp: 'AlexandreBergel 9/12/2012 15:40'!
add: process
	self assert: [ process isKindOf: Process ].
	processesToDispatch add: process! !

!LoomSimulatedchedulingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:40'!
getNextProcessToRun
	^ processesToDispatch removeFirst! !

!LoomSimulatedchedulingStrategy methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 15:39'!
initialize 
	super initialize.
	processesToDispatch := OrderedCollection new! !

LoomAcquiringProcessStrategy subclass: #LoomSmalltalkSchedulingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSmalltalkSchedulingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:38'!
getNextProcessToRun
	^ Processor nextReadyProcess! !

LoomObject subclass: #LoomAcquiringThreadStrategy
	instanceVariableNames: 'profiledThreads profiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringThreadStrategy commentStamp: 'AlexandreBergel 9/12/2012 15:33' prior: 0!
LoomAcquiringThreadStrategy is a generic way to obtain loom threads from the system. They may either be user selected, or programmatically selected.!

!LoomAcquiringThreadStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:31'!
forProfiler: aProfiler
	^ super new profiler: aProfiler; yourself! !

!LoomAcquiringThreadStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:31'!
new
	self error: 'Should not be used'! !

!LoomAcquiringThreadStrategy methodsFor: 'util' stamp: 'AlexandreBergel 9/12/2012 14:42'!
getLoomThreadsFrom: processes
	"Return a collection of LoomThreads from a list of Smalltalk processes"

	| id loomThreads newProcesses |
	id := 0.
	loomThreads := profiledThreads select: [ :loomThread | 
							id := id max: loomThread id.
							processes includes: loomThread process ].
	
	"processes that were not profiled before (i.e., have no loom thread associated to them)"
	newProcesses := processes copyWithoutAll: (loomThreads collect: #process). 
	
	loomThreads addAll: 
		(newProcesses collect: [ :p |
			id := id + 1.
			LoomThread new
				process: p;
				id: id;
				profiler: profiler;
				yourself ]).

	^ profiledThreads := loomThreads! !

!LoomAcquiringThreadStrategy methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:13'!
getThreadFrom: aSmalltalkProcess
	"Return the loom thread associated to the smalltalk process"
	
	^ self profiledThreads detect: [ :t | t process == aSmalltalkProcess ]! !

!LoomAcquiringThreadStrategy methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 14:40'!
getThreadFrom: aSmalltalkProcess ifAbsent: aBlock
	"Return the loom thread associated to the smalltalk process"

	"Not sure whether this method is useful at all"
	^ self profiledThreads detect: [ :t | t process == aSmalltalkProcess ] ifNone: aBlock! !

!LoomAcquiringThreadStrategy methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 13:09'!
initialize

	profiledThreads := OrderedCollection new! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 09:04'!
numberOfThreads
	"Return the number of threads profiled"

	^ profiledThreads size! !

!LoomAcquiringThreadStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:55'!
processesToProfile
	"Return the list of Smalltalk processes to profile"

	self subclassResponsibility ! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 9/12/2012 09:11'!
profiledThreads 
	"Return the list of loom processes for the profiled Smalltalk processes"
	
	^ self getLoomThreadsFrom: self processesToProfile! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 11:22'!
profiler: aProfiler
	profiler := aProfiler ! !

LoomAcquiringThreadStrategy subclass: #LoomAllThreadsStrategy
	instanceVariableNames: 'processesToProfile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAllThreadsStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:57'!
processesToProfile
 	"Maybe a weak collection would work better, no idea so far. Let's wait for a user to complain :-)"
	
	^" WeakSet withAll:"
		(Process allInstances
				reject: [ :each | each isTerminated ]).! !

LoomAcquiringThreadStrategy subclass: #LoomSelectiveStrategy
	instanceVariableNames: 'processesToProfile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSelectiveStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:01'!
for: processes
	^ super new processesToProfile: processes; yourself! !

!LoomSelectiveStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:37'!
forProcesses: processCollection profile: profiler.
	^ (super forProfiler: profiler) processesToProfile: processCollection; yourself! !

!LoomSelectiveStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:01'!
new
	self error: 'should not be used, use for: instead'! !

!LoomSelectiveStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:55'!
processesToProfile
	"Smalltalk processes"
	
	^ processesToProfile! !

!LoomSelectiveStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 08:52'!
processesToProfile: processes
	"Smalltalk processes as input"
	
	processesToProfile := processes! !

!LoomObject class methodsFor: 'instance creation' stamp: 'AlexandreBergel 9/6/2012 09:34'!
new
	^ self basicNew initialize! !

!LoomObject methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/6/2012 09:34'!
initialize! !

LoomObject subclass: #LoomProfiler
	instanceVariableNames: 'beginingProfilingTime profilingTime profilingProcess numberOfSamplings announcer acquiringThreadStrategy schedulingStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 09:17'!
allCurrentProcesses 
	"Return a weak array of processes"
	
"METHOD TO REMOVE!!!!"

	^" WeakSet withAll:"
		(Process allInstances
				reject: [:each | each isTerminated]).! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/6/2012 09:02'!
allCurrentThreads
	"Return a weak array of loom threads"

	^ self getLoomThreadsFrom: self allCurrentProcesses! !

!LoomProfiler methodsFor: 'events' stamp: 'AlexandreBergel 9/11/2012 22:04'!
announce: anEvent

	self announcer announce: anEvent! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/11/2012 21:44'!
announcer
	^ announcer ifNil: [ announcer := Announcer new ]! !

!LoomProfiler methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/5/2012 11:58'!
consumptionTimeOf: aSmalltalkProcess
	^ ((self getThreadFrom: aSmalltalkProcess) tick / numberOfSamplings) asFloat! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 15:10'!
createProfilingThread
	profilingProcess := [ [ self isProfiling ] whileTrue: [ 
							self sampleExecution.
							(Delay forMilliseconds: 5) wait.
						 ] ] forkAt: 72.
! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 09:25'!
getLoomThreadsFrom: processes
	"Return a collection of LoomThreads from a list of Smalltalk processes"

	"Maybe this method can be removed on some point...."
	| loomThreads |
	loomThreads := acquiringThreadStrategy getLoomThreadsFrom: processes.
	loomThreads do: [ :loomThread | loomThread profiler: self ].
	^ loomThreads! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 15:32'!
getNextProcessToRun
	^ Processor nextReadyProcess! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:02'!
getThreadFrom: aSmalltalkProcess
	"Return the loom thread associated to the smalltalk process"
	
	^ acquiringThreadStrategy getThreadFrom: aSmalltalkProcess! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:03'!
getThreadFrom: aSmalltalkProcess ifAbsent: aBlock
	"Return the loom thread associated to the smalltalk process"
	
	^ acquiringThreadStrategy getThreadFrom: aSmalltalkProcess ifAbsent: aBlock! !

!LoomProfiler methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 15:43'!
initialize
	super initialize.
	profilingTime := 0.
	beginingProfilingTime := 0.
	numberOfSamplings := 1.
	acquiringThreadStrategy := LoomAllThreadsStrategy forProfiler: self.
	schedulingStrategy := LoomSmalltalkSchedulingStrategy new.! !

!LoomProfiler methodsFor: 'testing' stamp: 'AlexandreBergel 9/5/2012 11:41'!
isProfiling
	"Are we profiling?"
	
	^ beginingProfilingTime ~= 0! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 19:32'!
numberOfSamplings
	^ numberOfSamplings! !

!LoomProfiler methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/12/2012 09:04'!
numberOfThreads
	"Return the number of threads profiled"
	
	^ acquiringThreadStrategy numberOfThreads! !

!LoomProfiler methodsFor: 'events' stamp: 'AlexandreBergel 9/11/2012 22:35'!
on: anEvent do: aBlock
	"Register the callback one arg aBlock for the event anEvent, a subclass of LoomEvent"
	
	self announcer when: anEvent do: aBlock! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:37'!
profile: processCollection
	"profile a collection of Smalltalk processes"
	
	acquiringThreadStrategy := LoomSelectiveStrategy forProcesses: processCollection profile: self.
	self startProfiling! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 14:47'!
profileAllProcesses
	"profile all the active processes"
	
	acquiringThreadStrategy := LoomAllThreadsStrategy forProfiler: self.
	self startProfiling! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:36'!
profilingTime
	^ profilingTime! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 15:32'!
sampleExecution
	"Simply increases the tick of the loom thread next to be executed"

	| loomThread nextProcessToRun |
	nextProcessToRun := self getNextProcessToRun.
	self announce: LoomSamplingEvent.
	nextProcessToRun isNil
		ifTrue: [ ^ self ].
	loomThread := self getThreadFrom: nextProcessToRun ifAbsent: [ ^ self ].
	loomThread increaseTick.
	numberOfSamplings := numberOfSamplings + 1.
	self announce: LoomSampledEvent! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/11/2012 21:42'!
startProfiling
	"Start the monitoring"
	
	self startTimer.
	self createProfilingThread! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/6/2012 09:50'!
startTimer
	beginingProfilingTime := LoomTime currentTimeAsMilliseconds! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/11/2012 21:42'!
stopProfiling
	"Stop the monitoring"
	
	profilingTime := LoomTime currentTimeAsMilliseconds - beginingProfilingTime.
	beginingProfilingTime := 0! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:02'!
threadsDo: aBlock
	"Evaluate a block for each loom thread"
	
	acquiringThreadStrategy profiledThreads do: aBlock! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 15:16'!
threadsDo: aBlock using: sortingStrategy
	"Evaluate a block for each loom thread"
	
	(sortingStrategy sort: acquiringThreadStrategy profiledThreads) do: aBlock! !

LoomObject subclass: #LoomSortingThreadStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomSortingThreadStrategy subclass: #LoomConsumptionSortingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomConsumptionSortingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:15'!
sort: loomThreads
	^ loomThreads asSortedCollection: [ :t1 :t2 | t1 consumptionTime > t2 consumptionTime ]! !

LoomObject subclass: #LoomThread
	instanceVariableNames: 'process id tick profiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomThread methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/5/2012 19:40'!
consumptionTime
	^ (((tick / profiler numberOfSamplings) * 100) asInteger / 100) asFloat! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
id
	"Return the id of the thread. If 0, then it means that none has been set"

	^ id! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
id: aNumber
	"Set the id number"
	
	id := aNumber! !

!LoomThread methodsFor: 'public' stamp: 'AlexandreBergel 9/5/2012 11:50'!
increaseTick
	tick := tick + 1! !

!LoomThread methodsFor: 'initialize' stamp: 'AlexandreBergel 9/5/2012 11:50'!
initialize
	super initialize.
	id := 0.
	tick := 0! !

!LoomThread methodsFor: 'testing' stamp: 'AlexandreBergel 9/6/2012 08:45'!
isSuspended
	^ process isSuspended! !

!LoomThread methodsFor: 'testing' stamp: 'AlexandreBergel 9/6/2012 08:45'!
isTerminated
	"Return true if the process is terminated"
	
	^ process isTerminated! !

!LoomThread methodsFor: 'printing' stamp: 'AlexandreBergel 9/6/2012 09:39'!
printOn: stream
	stream nextPutAll: '['.
	stream nextPutAll: (self consumptionTime * 100) asInteger printString.
	stream nextPutAll: '%] ('.
	stream nextPutAll: self priority printString.
	stream nextPutAll: ')'.
	
	self isTerminated 
		ifTrue: [ stream nextPutAll: ' T '].
	
	(process suspendedContext class ==  MethodContext)
		ifTrue: [ stream nextPutAll: process suspendedContext home printString ]
		ifFalse: [ stream nextPutAll: process suspendedContext printString.]
	! !

!LoomThread methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/6/2012 08:42'!
priority
	"Answer the priority of the receiver."

	^ process priority! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
process
	^ process! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
process: anObject
	process := anObject! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 09:21'!
profiler
	^ profiler! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 19:29'!
profiler: aProfiler
	profiler := aProfiler! !

!LoomThread methodsFor: 'public' stamp: 'AlexandreBergel 9/6/2012 09:15'!
resetTick
	"Invoked after each sampling"
	tick := 0! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 11:51'!
tick
	^ tick! !
